<?php

namespace Drupal\eus\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Session\SessionManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ConfirmRoleEndSessionForm.
 */
class ConfirmRoleEndSessionForm extends ConfirmFormBase {

  /**
   * Drupal\Core\Session\SessionManagerInterface definition.
   *
   * @var \Drupal\Core\Session\SessionManagerInterface
   */
  protected $sessionManager;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * ID of the item to end session.
   *
   * @var string
   */
  protected $role;

  /**
   * Use core services object.
   */
  public function __construct(SessionManagerInterface $session_manager, EntityTypeManagerInterface $entity_type_manager) {
    $this->sessionManager = $session_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('session_manager'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return "confirm_eus_role_form";
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    $name = $this->entityTypeManager->getStorage('user_role')->load($this->role)->label();
    return $this->t('Are you sure you want to destroy all users session associated with %name role?', ['%name' => $name]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('entity.user_role.collection');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Destroy session');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $role = NULL) {
    $this->role = $role;
    $roles = $this->entityTypeManager->getStorage('user_role')->loadMultiple();

    if (!array_key_exists($this->role, $roles)) {
      $this->messenger()->addError($this->t('Invalid role found.'));
      return;
    }

    $users = $this->getUsersOfRole($this->role);
    if (empty($users)) {
      $this->messenger()->addError($this->t('No users associated with %name role.', ['%name' => $this->role]));
      return;
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $users = $this->getUsersOfRole($this->role);
    foreach ($users as $id) {
      $this->sessionManager->delete($id);
    }

    $this->messenger()->addMessage($this->t('Session has been destroyed for all users associated with %name role.', ['%name' => $this->role]));
    $form_state->setRedirect('entity.user_role.collection');
  }

  /**
   * Provide users associated with role.
   *
   * @param string $role
   *   String of role.
   *
   * @return nixed
   *   An array of user ids.
   */
  public function getUsersOfRole($role) {
    $ids = $this->entityTypeManager->getStorage('user')->getQuery()
      ->condition('status', 1)
      ->condition('roles', $this->role)
      ->execute();
    return $ids;
  }

}
